"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice_py.types import (
    BaseModel,
    dto_updateinvoicerequest as types_dto_updateinvoicerequest,
)
from flexprice_py.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class UpdateInvoiceRequestTypedDict(TypedDict):
    id: str
    r"""Invoice ID"""
    body: types_dto_updateinvoicerequest.DtoUpdateInvoiceRequestTypedDict
    r"""Invoice Update Request"""


class UpdateInvoiceRequest(BaseModel):
    id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""Invoice ID"""

    body: Annotated[
        types_dto_updateinvoicerequest.DtoUpdateInvoiceRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]
    r"""Invoice Update Request"""
