"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, Optional
from typing_extensions import NotRequired, TypedDict


class DtoCreateCreditNoteLineItemRequestTypedDict(TypedDict):
    amount: str
    r"""amount is the monetary amount to be credited for this line item"""
    invoice_line_item_id: str
    r"""invoice_line_item_id is the unique identifier of the invoice line item being credited"""
    display_name: NotRequired[str]
    r"""display_name is an optional human-readable name for this credit note line item"""
    metadata: NotRequired[Dict[str, str]]


class DtoCreateCreditNoteLineItemRequest(BaseModel):
    amount: str
    r"""amount is the monetary amount to be credited for this line item"""

    invoice_line_item_id: str
    r"""invoice_line_item_id is the unique identifier of the invoice line item being credited"""

    display_name: Optional[str] = None
    r"""display_name is an optional human-readable name for this credit note line item"""

    metadata: Optional[Dict[str, str]] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["display_name", "metadata"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m
